package com.jz.sms.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.jz.sms.chuanglan.dto.CLSendFixedResponse;
import com.jz.sms.chuanglan.dto.CLTemplateResponse;
import com.jz.sms.chuanglan.service.ChuangLanSmsService;
import com.jz.sms.config.ResponseCode;
import com.jz.sms.dto.*;
import com.jz.sms.repository.domain.SmsTemplateInfo;
import com.jz.sms.repository.domain.SysBatchInfo;
import com.jz.sms.service.MessageService;
import com.jz.sms.service.SmsTemplateService;
import com.jz.sms.service.bean.TemplateChangeBean;
import com.jz.sms.util.id.IdHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Service
public class MessageServiceImpl implements MessageService {

    private static final Logger log = LoggerFactory.getLogger(MessageServiceImpl.class);

    @Autowired
    private SysBatchServiceImpl sysBatchService;

    @Autowired
    private SmsTemplateService smsTemplateService;

    @Autowired
    private ChuangLanSmsService chuangLanSmsService;

    /**
     * 短信模板创建
     *
     * @param requestDTO
     * @return
     */
    @Override
    public DmHubTemplateResponse createTemplate(DmHubTemplateRequest requestDTO) {
        log.info("进入创建短信模板业务逻辑，入参：{}", requestDTO);

        DmHubTemplateResponse responseDTO = new DmHubTemplateResponse("200", "成功");
        try {
            String templateContent = requestDTO.getTemplateContent();
            String signature = requestDTO.getSignature();
            String smsType = requestDTO.getSmsType();
            String tenantId = requestDTO.getTenantId();
            String templateId = requestDTO.getTemplateId();
            String templateName = requestDTO.getTemplateName();

            // 替换模板格式，保存适合创蓝发送短信的模板，避免发送时修改模板格式
            TemplateChangeBean bean = dmHub2ChuangLanSend(templateContent);
            String chuanglanSend = bean.getNewStr();
            String params = bean.getParams().toString();

            SmsTemplateInfo info = new SmsTemplateInfo();
            info.setContent(templateContent);
            info.setType(smsType);
            info.setDmTemplateId(templateId);
            info.setTenantId(tenantId);
            info.setId(IdHandler.nextId());
            info.setSignature(signature);
            info.setTemplateName(templateName);
            info.setDateCreated(new Date());
            info.setLastUpdated(new Date());
            info.setLastUpdated(new Date());
            info.setUpContent(chuanglanSend);
            info.setParams(params);
            boolean insert = smsTemplateService.insert(info);
            if (!insert) {
                return new DmHubTemplateResponse("500", "系统异常");
                //TODO 发送告警
            }
            String chaunglanCreate = dmHub2ChuangLanCreate(chuanglanSend);
            CLTemplateResponse template = chuangLanSmsService.createTemplate(chaunglanCreate, smsType);
            if (ResponseCode.SUCCESS.getCode().equals(template.getCode())) {
                String id = template.getId();
                SmsTemplateInfo updateInfo = new SmsTemplateInfo();
                updateInfo.setId(info.getId());
                updateInfo.setUpTemplateId(id);
                smsTemplateService.updateById(updateInfo);
                return responseDTO;
            } else {
                //TODO 发送告警，模板添加失败。

                return new DmHubTemplateResponse(template.getCode(), template.getMsg());
            }
        } catch (Exception exception) {
            log.error("错误信息", exception);
            responseDTO = new DmHubTemplateResponse("500", "系统异常");
        }
        return responseDTO;

    }


    /**
     * 将DM HUb的模板装成可以直接用在创蓝发送短信时的模板
     *
     * @param content
     * @return
     */
    public TemplateChangeBean dmHub2ChuangLanSend(String content) {
        String regex = "\\$\\{[^}]+\\}";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(content);

        List<String> groups = new ArrayList<>();
        while (matcher.find()) {
            String group = matcher.group();
            groups.add(group);
        }
        if (groups.isEmpty()) {
            return new TemplateChangeBean(content, groups);
        }
        int idx = content.indexOf("${surl!'");
        if (idx >= 0) {
            String substring = content.substring(idx);
            System.out.println(substring);
            int i1 = substring.indexOf("'}");
            System.out.println(i1);
            String substring1 = substring.substring(0, i1 + 2);
            System.out.println(substring1);
            System.out.println(content);
            substring1 = Matcher.quoteReplacement(substring1);
            substring1 = escapeRegex(substring1);
            content = content.replaceAll(substring1, "\\{\\$var\\}");
        }

        for (String g : groups) {
            String t = g;
            g = Matcher.quoteReplacement(g);
            g = escapeRegex(g);

            content = content.replaceAll(g, "\\{\\$var\\}");
        }
        return new TemplateChangeBean(content, groups);
    }


    public String dmHub2ChuangLanCreate(String content) {
        content = content.replaceAll("\\{\\$var\\}", "\\{s20\\}");
        return content;

    }

    public static String escapeRegex(String s) {
        String[] symbols = new String[]{"\\/", "\\[", "\\]", "\\(", "\\)", "\\{", "\\}", "\\?", "\\+", "\\*", "\\|", "\\.", "\\^", "\\-", "\\^"};
        if (s != null) {
            for (String sym : symbols) {
                s = s.replaceAll(sym, "\\" + sym);
            }
        }
        return s;
    }

    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public DmHubSendResponse send(DmHubSendRequest requestDTO) {

        try {
            String dmHubBatchId = requestDTO.getBatchId();
            String templateId = requestDTO.getTemplateId();
            String audienceIdType = requestDTO.getAudienceIdType();
            JSONObject data = requestDTO.getData();
            // 1、获取模板信息
            SmsTemplateInfo templateInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("dm_template_id", dmHubBatchId));
            if (null == templateInfo) {
                return null;
            }

            // 2、根据请求信息获取用户的手机号码（此处不考虑配置DMHUB系统不是SMS的情况），直接拿请求体中的_audienceId字段值，该值为手机号码
            String phone = data.getString("_audienceId");
            List<String> phoneList = new ArrayList<>();
            phoneList.add(phone);
            String type = templateInfo.getType();

            // 3、记录批次信息
            SysBatchInfo sysBatchInfo = new SysBatchInfo();
            sysBatchInfo.setBatchId(IdHandler.nextId());
            sysBatchInfo.setDmBatchId(dmHubBatchId);
            sysBatchInfo.setId(IdHandler.nextId());
            sysBatchInfo.setDmTemplateId(templateId);
            sysBatchInfo.setReport("1");
            sysBatchInfo.setSmsNum(1);
            sysBatchInfo.setDateCreated(new Date());
            sysBatchInfo.setLastUpdated(new Date());
            boolean insert = sysBatchService.insert(sysBatchInfo);


            // 判断模板是固定短信还是变量短信
            String content = templateInfo.getContent();
//            String content2 = "亲爱的${name@6!\"\"}，您的手机号码：${mobile@12!\"\"}参与了优惠活动，详见： ${surl!'https://www.baidu.com/?cl_sr=短消息'} 回TD退订";

            String regex = "\\$\\{[^}]+\\}";
            Pattern pattern = Pattern.compile(regex);
            Matcher matcher = pattern.matcher(content);

            List<String> list = new ArrayList<>();

            while (matcher.find()) {
                String group = matcher.group();
                list.add(group);

            }
            log.info("模板变量参数为：{}", list.toString());
            if (list.size() > 0) {
                // 变量模板
                // 3、模板参数文本替换。获取模板中的动态参内容，替换成创蓝需要的格式${var}.将请求中的参数信息和模板中的变量名对应起来，封装创蓝请求参数
                for (String str : list) {

                }
                CLSendFixedResponse chuanglanResponse = chuangLanSmsService.sendByFixed(dmHubBatchId, content, phoneList, type);
            } else {
                // 固定模板

                // 4、封裝數據
                chuangLanSmsService.sendByVariable(dmHubBatchId, content, phoneList, type);
            }


        } catch (Exception e) {

        }

        return null;
    }

    @Override
    public DmHubBatchSendResponse batchSend(DmHubBatchSendRequest request) {

        String batchId = request.getBatchId();
        String templateId = request.getTemplateId();
        String audienceIdType = request.getAudienceIdType();
        List<JSONObject> data = request.getData();

        SmsTemplateInfo templateInfo = smsTemplateService.selectOne(new EntityWrapper<SmsTemplateInfo>().eq("dm_template_id", templateId));
        if (null == templateInfo) {
            return new DmHubBatchSendResponse("500", "模板不存在");
        }

        for (JSONObject json : data) {
            Set<String> keys = json.keySet();
            StringBuilder sb = new StringBuilder();
            String mobile = json.getString("_audienceId");
            json.remove("name");
            json.remove("id");
            json.remove("_audienceId");
            sb.append(mobile);
            if (!json.isEmpty()) {
                for (String key : keys) {
                    String value = json.getString(key);

                }
            }
        }


        return null;
    }


}


